import { Canvas } from '@nativescript-community/ui-canvas';
import { ImageSource, Observable, Screen } from '@nativescript/core';
import { ImageBitmapRenderingContext } from '../contexts/ImageBitmapRenderingContext';
import { OffscreenCanvasRenderingContext2D } from '../contexts/OffscreenCanvasRenderingContext2D';
class NSOffscreenCanvas extends Observable {
    constructor(width, height) {
        super();
        this._width = width;
        this._height = height;
        this._updateNativeContext();
    }
    _updateNativeContext() {
        const scale = Screen.mainScreen.scale;
        if (this._nativeContext != null) {
            this._nativeContext.release();
        }
        this._nativeContext = new Canvas(this._width * scale, this._height * scale);
        this._nativeContext.scale(scale, scale);
    }
    getContext(contextId, contextAttributes) {
        const self = this;
        let cl;
        switch (contextId) {
            case '2d':
                cl = OffscreenCanvasRenderingContext2D;
                break;
            case 'bitmaprenderer':
                cl = ImageBitmapRenderingContext;
                break;
            default:
                cl = null;
                break;
        }
        if (cl == null) {
            return null;
        }
        if (this._context != null && this._context instanceof cl) {
            return this._context;
        }
        const context = new cl();
        Object.defineProperty(context, 'canvas', {
            get() {
                return self;
            },
        });
        this._context = context;
        return context;
    }
    transferToImageBitmap() {
        return new ImageSource(this._nativeContext.getImage());
    }
    convertToBlob(options) {
        console.warn('Method convertToBlob is not implemented');
        return null;
    }
    isOffscreenBufferEnabled() {
        return true;
    }
    get nativeContext() {
        return this._nativeContext;
    }
    get width() {
        return this._width;
    }
    set width(val) {
        const oldVal = this._width;
        this._width = val;
        if (this._width !== oldVal) {
            this._updateNativeContext();
        }
    }
    get height() {
        return this._height;
    }
    set height(val) {
        const oldVal = this._height;
        this._height = val;
        if (this._height !== oldVal) {
            this._updateNativeContext();
        }
    }
}
export { NSOffscreenCanvas as OffscreenCanvas };
//# sourceMappingURL=OffscreenCanvas.js.map